/*
 * ForceField.h
 *
 * Copyright Notice: see copyright.txt
 *
 *  Date: 3/8/2012 (D/M/Y)
 *  Author: Dmitrij Lioubartsev
 *  Email: dmitrijl42@gmail.com
 *
 * This class reads and parses a Force Field File, stores
 * the relevant information from it and allows for extraction
 * of atomic data. It also has a method for automatically
 * set that data for all atoms.
 * For information about what a force field file is, see readme.txt.
 */

#ifndef FORCEFIELD_H_
#define FORCEFIELD_H_

#include <vector>
#include <string>
#include <set>
#include <map>
#include <fstream>
#include <cstdlib>
#include <cmath>
#include <stdexcept>
#include <iostream>
using namespace std;

#include "Atom.h"
#include "ForceFieldJoint.h"

class ForceField {
private:

	// The variable that stores the data about atoms.
	map<string, vector<ForceFieldJoint> > ff;

	// a set of all the atom types that appear in the allAtoms parameter.
	set<string> atomTypes;

public:

	ForceField(vector<Atom>* allAtoms);
	virtual ~ForceField();

	bool loadFromFile(string ff_file);
	void setCharge(double total_Q, vector<Atom>* atoms, bool equalize);

	ForceFieldJoint::data getData(Atom& a);


private: //private methods:
	//these are for the file reading
	bool skipWhiteSpace(ifstream& stream);
	bool getWord(ifstream& stream, string& word);
	bool isWhiteSpace(char c);

};

#endif /* FORCEFIELD_H_ */
