/*
 * ForceFieldJoint.h
 *
 * Copyright Notice: see copyright.txt
 *
 *  Date: 3/8/2012 (D/M/Y)
 *  Author: Dmitrij Lioubartsev
 *  Email: dmitrijl42@gmail.com
 *
 *  This is a storage class for one joint (line basically) in the
 *  force field file. It holds info of the element, the bonds and
 *  the corresponding charge and force field type, and also provides a method
 *  for checking if an atom matches it.
 */

#ifndef FORCEFIELDJOINT_H_
#define FORCEFIELDJOINT_H_

#include <string>
#include <vector>
using namespace std;

#include "Atom.h"

class ForceFieldJoint {
public: //struct definitions
	struct data {
		double charge;
		string fft; //force field type
	};

private: //fields
	string element;
	int numBonds;	//amount of bonds. Essentially bondTypes.size()
	vector<string> bondTypes;
	data dat;

public: //public methods


	ForceFieldJoint(string element, vector<string>* bonds, data dat);
	virtual ~ForceFieldJoint();

	//getters
	string getElement();
	int getBondCount();
	data getData();

	bool matches(vector<Atom*>* structure);
};

#endif /* FORCEFIELDJOINT_H_ */
